-- tictactoe domain: environment plays as O (goes second)



-- program options
debug_mode = false;

-- initialise prng
seed = os.time()
math.randomseed(seed)
math.random(); math.random(); math.random();

-- encoding/decoding tables
encode_cell = {}
encode_cell['.'] = '00'
encode_cell['X'] = '01'
encode_cell['O'] = '11'

encode_rew = {}
encode_rew[0] = '000' -- illegal move      -3
encode_rew[1] = '001' -- loss              -2
encode_rew[3] = '011' -- game continuing   +0
encode_rew[4] = '100' -- draw              +1
encode_rew[5] = '101' -- win               +2

decode_act = {}
decode_act['0000'] = '11'
decode_act['0001'] = '12'
decode_act['0010'] = '13'
decode_act['0011'] = '21'
decode_act['0100'] = '22'
decode_act['0101'] = '23'
decode_act['0110'] = '31'
decode_act['0111'] = '32'
decode_act['1000'] = '33'

-- global game board
board = {}
for i=1,3 do
	board[i] = {}
end



-- reset the game board (world state)
function clear_board ()

	for i=1,3 do
		for j=1, 3 do
			board[i][j] = '.'
		end
	end
end


-- encode the entire board state into an observation
function encode_board ()

	local rval = ''

	for i=1,3 do
		for j=1, 3 do
			rval = rval .. encode_cell[board[i][j]]
		end
	end

	return rval
end


-- is the entire board filled?
function board_filled ()

	local filled = true

	for i=1,3 do
		for j=1,3 do
			filled = filled and board[i][j] ~= '.'
		end
	end

	return filled
end


-- is the game over?
function game_over ()
	return winning_side() ~= nil or board_filled()
end


-- compute the winning side, nil is returned in case of no winner
function winning_side ()

	-- handle rows
	for i=1,3 do
		if (board[i][1] ~= '.' and board[i][1] == board[i][2] and board[i][2] == board[i][3]) then
			return board[i][1]
		end
	end

	-- handle columns
	for j=1,3 do
		if (board[1][j] ~= '.' and board[1][j] == board[2][j] and board[2][j] == board[3][j]) then
			return board[1][j]
		end
	end

	-- handle diagonals
	if (board[1][1] ~= '.' and board[1][1] == board[2][2] and board[2][2] == board[3][3]) then
		return board[1][1]
	end
	if (board[1][3] ~= '.' and board[1][3] == board[2][2] and board[2][2] == board[3][1]) then
		return board[1][3]
	end

	return nil
end


-- compute the reward for an agent based on the world state
function compute_reward ()

	if (last_move_illegal) then
		return 0
	elseif (winner == 'O') then
		return 1
	elseif (winner == 'X') then
		return 5
	elseif (winner == 'draw') then
		return 4
	else  -- game continuing
		return 3
	end
end


-- compute the next observation based on the world state
function compute_observation ()

	return encode_board()
end


-- compute the winner if the game is over
function compute_winner ()

	if (winning_side() == nil) then
		winner = 'draw'
	else
		winner = winning_side()
	end
end


-- determine the next state
function compute_next_state (action)

	-- extract move
	local row = tonumber(string.sub(action, 1, 1))
	local col = tonumber(string.sub(action, 2, 2))

	-- update state with agent's move
	last_move_illegal = board[row][col] ~= '.'
	if (last_move_illegal) then
		clear_board()
		return
	end

	board[row][col] = 'X'
	last_move_illegal = false
	if (game_over()) then
		compute_winner()
		clear_board()
		return
	end

	-- play a legal, random move
	repeat
		row = math.random(3)
		col = math.random(3)
	until board[row][col] == '.'

	board[row][col] = 'O'
	if (game_over()) then
		compute_winner()
		clear_board()
		return
	end

	winner = nil

end


-- print out a game board
function print_board ()
	for i=1,3 do
		for j=1,3 do
			io.write(board[i][j])
		end
		io.write('\n')
	end
end


-- debugging print
function dprint (str)

	if (debug_mode) then
		local f = io.output()
		io.output(stderr)
		print(str)
		io.output(f)
	end
end



-- main loop
line   = ''
clear_board()
last_move_illegal = false
winner = nil

repeat

	if (debug_mode) then print_board() end

	local obs = compute_observation()
	local rew = compute_reward()

	dprint('obs=' .. obs .. ', rew=' .. rew)

	local enc_rew = encode_rew[rew]

	print(obs .. enc_rew)
	io.flush()


	-- grab action
	line = io.read()
	if (line == '') then break end
	action = decode_act[line];
	dprint('action=' .. action)

	compute_next_state(action)


until line == ''




