
-- program options
debug_mode = false;

-- initialise prng
seed = os.time()
math.randomseed(seed)
math.random(); math.random(); math.random();

-- encoding/decoding tables
encode_obs = {}
encode_obs.null  = '00'
encode_obs.left  = '10'
encode_obs.right = '11'

encode_rew = {}
encode_rew.null = '1100100'
encode_rew[0]   = '0000000'
encode_rew[99]  = '1100011'
encode_rew[110] = '1101110'

decode_act = {}
decode_act['00'] = 'listen'
decode_act['01'] = 'left'
decode_act['10'] = 'right'


-- reset the POMDP to the starting state
function assign_tiger ()
	local r = math.random()
	if (r < 0.5) then
		tiger_state = 'left'
		safe_state  = 'right'
	else
		tiger_state = 'right'
		safe_state  = 'left'
	end
end


-- compute the reward for an agent given an action
function compute_reward (action)

	if (action == tiger_state) then
		-- we get eaten
		return 0
	elseif (action == 'listen') then
		return 99
	else -- we open the right door
		return 110
	end
end


-- compute the next observation for an agent given an action
function compute_observation (action)

	if (action == 'listen') then
		if (math.random() < 0.85) then
			return tiger_state
		else
			return safe_state
		end
	end

	return 'null'
end


-- debugging print
function dprint (str)

	if (debug_mode) then
		local f = io.output()
		io.output(stderr)
		print(str)
		io.output(f)
	end
end


-- main loop: play the tiger POMDP
--            until a blank line is seen

action = 'left'
line   = ''

-- display dummy observation/reward
print(encode_obs.null .. encode_rew.null)
io.flush()

repeat

	if (action == 'left' or action == 'right') then
		-- reset game
		assign_tiger()
		dprint('tiger=' .. tiger_state)
	end

	-- grab action
	line = io.read()
	if (line == '') then break end

	action = decode_act[line];
	dprint('action=' .. action)
	local obs = compute_observation(action)
	local rew = compute_reward(action)

	dprint('obs=' .. obs .. ', rew=' .. rew)

	local enc_obs = encode_obs[obs]
	local enc_rew = encode_rew[rew]

	print(enc_obs .. enc_rew)
	io.flush()


until line == ''




